from io import BytesIO
from pathlib import Path
from typing import Union
from base64 import b64encode

from PIL import Image


async def convert_img(
    img: Union[Image.Image, str, Path, bytes], is_base64: bool = False
):
    """
    :说明:
      将PIL.Image对象转换为bytes或者base64格式。
    :参数:
      * img (Image): 图片。
      * is_base64 (bool): 是否转换为base64格式, 不填默认转为bytes。
    :返回:
      * res: bytes对象或base64编码图片。
    """
    if isinstance(img, Image.Image):
        img = img.convert('RGB')
        result_buffer = BytesIO()
        img.save(result_buffer, format='PNG', quality=80, subsampling=0)
        res = result_buffer.getvalue()
        if is_base64:
            res = 'base64://' + b64encode(res).decode()
        return res
    elif isinstance(img, bytes):
        return 'base64://' + b64encode(img).decode()
    else:
        return f'[CQ:image,file=file:///{str(img)}]'




from pathlib import Path
import json
import yaml
import sqlite3

# DATABASE = Path() / "data" / "xiuxian"
DATABASE = Path() / "xiuxian"

class XiuConfig:

    def __init__(self):
        self.config_jsonpath = DATABASE / "config.json"


    def read_data(self):
        """配置数据"""
        with open(self.config_jsonpath, 'r', encoding='utf-8') as e:
            data = json.load(e)
            print(data)
            return data

    def write_data(self, user_id):
        json_data = self.read_data()
        json_data[user_id] = True
        with open(self.config_jsonpath, 'w+') as f:
            json.dump(json_data, f)


def get_sql():
    database_path = r"D:\yuzi_bot\yuzi_bot\data\xiuxian\xiuxian.db"
    conn = sqlite3.connect(database_path)
    check_sql = f"select goods_num from user_back where user_id=? and goods_id=?"
    cur = conn.cursor()
    cur.execute(check_sql, ('111', '1'))
    result = cur.fetchone()
    if result:
        goods_nums = result[0]
        sql = f"UPDATE user_back set num=?,update_time=? WHERE user_id=? and goods_id=?"
        cur.execute(sql, (user_id, goods_id, name, type_n, num, remake))
        conn.commit()
    else:
        sql = """
                INSERT INTO user_back ( user_id, goods_id, goods_name, goods_type, goods_num, create_time, update_time )
        VALUES (?,?,?,?,?,?,?)"""
        cur.execute(sql, (user_id, goods_id, name, type_n, num, remake))
        conn.commit()

    # sql = f"INSERT INTO back(user_id, goods_id, goods_name, goods_type, goods_num, create_time, update_time) VALUES (?,?,?,?,?,?)"
    # cur = self.conn.cursor()
    # cur.execute(sql, (user_id, goods_id, name, type_n, num, remake))
    # self.conn.commit()



if __name__ == '__main__':
    # XiuConfig().write_data("抢灵石")
    # DATA = XiuConfig().read_data()
    get_sql()



from .xiuxian_config import XiuConfig

"""
Help帮助信息

"""

__xiuxian_version__ = "v0.4.26"

__xiuxian_notes__ = f"""
指令：
1、我要修仙：进入修仙模式
2、我的修仙信息：获取修仙数据
3、修仙签到：获取灵石及修为
4、重入仙途：重置灵根数据，每次{XiuConfig().remake}灵石
5、金银阁：猜大小/数字，赌灵石 示例:金银阁10大/小/猜3
6、改名xx：修改你的道号
7、突破：修为足够后，可突破境界（一定几率失败）
8、闭关、出关、灵石出关：修炼增加修为，挂机功能
9、送灵石100@xxx,偷灵石@xxx,抢灵石@xxx
10、排行榜：修仙排行榜，灵石排行榜，战力排行榜，宗门排行榜
11、悬赏令：获取任务单，接取任务示例：悬赏令接取1， 结算命令示例：悬赏令结算
12、我的状态：查看当前HP,我的功法：查看当前技能
13、宗门系统：发送"宗门帮助"获取
14、钱庄系统：发送"钱庄帮助"获取
15、世界BOSS：发送"世界boss帮助"获取
16、功法：发送“我的功法”查看，当前获取途径看“宗门帮助”
17、启用/禁用修仙功能：当前群开启或关闭修仙功能
""".strip()


import datetime
import json
import random
from .xiuxian2_handle import XiuxianDateManage
from .read_buff import UserBuffDate


def Player_fight(player1: dict, player2: dict, type_in: 2):
    """
    回合制战斗
    type_in : 1-切磋，不消耗气血、真元
              2-战斗，消耗气血、真元
    数据示例：
    {"user_id": None,"道号": None, "气血": None, "攻击": None, "真元": None, '会心':None, 'exp':None}
    """
    user1buffdate = UserBuffDate(player1['user_id'])  # 1号的buff信息
    user1mainbuffdata = user1buffdate.get_user_main_buff_data()
    user1hpbuff = user1mainbuffdata['hpbuff'] if user1mainbuffdata != None else 0
    user1mpbuff = user1mainbuffdata['mpbuff'] if user1mainbuffdata != None else 0
    user2buffdate = UserBuffDate(player2['user_id'])  # 2号的buff信息
    user2mainbuffdata = user2buffdate.get_user_main_buff_data()
    user2hpbuff = user2mainbuffdata['hpbuff'] if user2mainbuffdata != None else 0
    user2mpbuff = user2mainbuffdata['mpbuff'] if user2mainbuffdata != None else 0

    # 有技能，则开启技能模式

    player1_skil_open = False
    player2_skil_open = False
    if user1buffdate.get_user_sec_buff_data() != None:
        user1skilldate = user1buffdate.get_user_sec_buff_data()
        player1_skil_open = True
    if user2buffdate.get_user_sec_buff_data() != None:
        user2skilldate = user2buffdate.get_user_sec_buff_data()
        player2_skil_open = True

    play_list = []
    suc = None
    isSql = False
    if type_in == 2:
        isSql = True
    user1turnskip = True
    user2turnskip = True
    player1turncost = 0  # 先设定为初始值 0
    player2turncost = 0  # 先设定为初始值 0
    player1js = 1  # 减伤率
    player2js = 1  # 减伤率
    user1skillsh = 0
    user2skillsh = 0
    user1buffturn = True
    user2buffturn = True
    while True:
        msg1 = "{}发起攻击，造成了{}伤害\n"
        msg2 = "{}发起攻击，造成了{}伤害\n"
        if player1_skil_open:  # 是否开启技能
            if user1turnskip:  # 无需跳过回合
                play_list.append(f"☆------{player1['道号']}的回合------☆")
                user1hpconst, user1mpcost, user1skill_type, skillrate = get_skill_hp_mp_data(player1, user1skilldate)
                if player1turncost == 0:  # 没有持续性技能生效
                    player1js = 1  # 没有持续性技能生效,减伤恢复
                    if isEnableUserSikll(player1, user1hpconst, user1mpcost, player1turncost,
                                         skillrate):  # 满足技能要求，#此处为技能的第一次释放
                        skillmsg, user1skillsh, player1turncost = get_skill_sh_data(player1, user1skilldate)
                        if user1skill_type == 1:  # 直接伤害类技能
                            play_list.append(skillmsg)
                            player1 = calculate_skill_cost(player1, user1hpconst, user1mpcost)

                            player2['气血'] = player2['气血'] - int(user1skillsh * player2js)  # 玩家1的伤害 * 玩家2的减伤
                            play_list.append(f"{player2['道号']}剩余血量{player2['气血']}")

                        elif user1skill_type == 2:  # 持续性伤害技能
                            play_list.append(skillmsg)
                            player1 = calculate_skill_cost(player1, user1hpconst, user1mpcost)
                            player2['气血'] = player2['气血'] - int(user1skillsh * player2js)  # 玩家1的伤害 * 玩家2的减伤
                            play_list.append(f"{player2['道号']}剩余血量{player2['气血']}")

                        elif user1skill_type == 3:  # buff类技能
                            user1buff_type = user1skilldate['bufftype']
                            if user1buff_type == 1:  # 攻击类buff
                                isCrit, player1_sh = get_turnatk(player1, user1skillsh)  # 判定是否暴击
                                if isCrit:
                                    msg1 = "{}发起会心一击，造成了{}伤害\n"
                                else:
                                    msg1 = "{}发起攻击，造成了{}伤害\n"
                                player1 = calculate_skill_cost(player1, user1hpconst, user1mpcost)
                                play_list.append(skillmsg)
                                play_list.append(msg1.format(player1['道号'], player1_sh))
                                player2['气血'] = player2['气血'] - int(player1_sh * player2js)  # 玩家1的伤害 * 玩家2的减伤
                                play_list.append(f"{player2['道号']}剩余血量{player2['气血']}")


                            elif user1buff_type == 2:  # 减伤类buff,需要在player2处判断
                                isCrit, player1_sh = get_turnatk(player1)  # 判定是否暴击
                                if isCrit:
                                    msg1 = "{}发起会心一击，造成了{}伤害\n"
                                else:
                                    msg1 = "{}发起攻击，造成了{}伤害\n"

                                player1 = calculate_skill_cost(player1, user1hpconst, user1mpcost)
                                play_list.append(skillmsg)
                                play_list.append(msg1.format(player1['道号'], player1_sh))
                                player2['气血'] = player2['气血'] - int(player1_sh * player2js)  # 玩家1的伤害 * 玩家2的减伤
                                play_list.append(f"{player2['道号']}剩余血量{player2['气血']}")
                                player1js = 1 - user1skillsh

                        elif user1skill_type == 4:  # 封印类技能
                            play_list.append(skillmsg)
                            player1 = calculate_skill_cost(player1, user1hpconst, user1mpcost)

                            if user1skillsh:  # 命中
                                user2turnskip = False
                                user2buffturn = False

                    else:  # 没放技能，打一拳
                        isCrit, player1_sh = get_turnatk(player1)  # 判定是否暴击
                        if isCrit:
                            msg1 = "{}发起会心一击，造成了{}伤害\n"
                        else:
                            msg1 = "{}发起攻击，造成了{}伤害\n"
                        play_list.append(msg1.format(player1['道号'], player1_sh))
                        player2['气血'] = player2['气血'] - int(player1_sh * player2js)  # 玩家1的伤害 * 玩家2的减伤
                        play_list.append(f"{player2['道号']}剩余血量{player2['气血']}")


                else:  # 持续性技能判断,不是第一次
                    if user1skill_type == 2:  # 持续性伤害技能
                        player1turncost = player1turncost - 1
                        skillmsg = get_persistent_skill_msg(player1['道号'], user1skilldate['name'], user1skillsh,
                                                            player1turncost)
                        play_list.append(skillmsg)
                        isCrit, player1_sh = get_turnatk(player1)  # 判定是否暴击
                        if isCrit:
                            msg1 = "{}发起会心一击，造成了{}伤害\n"
                        else:
                            msg1 = "{}发起攻击，造成了{}伤害\n"
                        play_list.append(msg1.format(player1['道号'], player1_sh))
                        # 玩家1的伤害 * 玩家2的减伤,持续性伤害不影响普攻
                        player2['气血'] = player2['气血'] - int((user1skillsh + player1_sh) * player2js)
                        play_list.append(f"{player2['道号']}剩余血量{player2['气血']}")


                    elif user1skill_type == 3:  # buff类技能
                        user1buff_type = user1skilldate['bufftype']
                        if user1buff_type == 1:  # 攻击类buff
                            isCrit, player1_sh = get_turnatk(player1, user1skillsh)  # 判定是否暴击

                            if isCrit:
                                msg1 = "{}发起会心一击，造成了{}伤害\n"
                            else:
                                msg1 = "{}发起攻击，造成了{}伤害\n"

                            player1turncost = player1turncost - 1
                            play_list.append(f"{user1skilldate['name']}增伤剩余:{player1turncost}回合")
                            play_list.append(msg1.format(player1['道号'], player1_sh))
                            player2['气血'] = player2['气血'] - int(player1_sh * player2js)  # 玩家1的伤害 * 玩家2的减伤
                            play_list.append(f"{player2['道号']}剩余血量{player2['气血']}")


                        elif user1buff_type == 2:  # 减伤类buff,需要在player2处判断
                            print("玩家1减伤" + str(player1js))
                            isCrit, player1_sh = get_turnatk(player1)  # 判定是否暴击
                            if isCrit:
                                msg1 = "{}发起会心一击，造成了{}伤害\n"
                            else:
                                msg1 = "{}发起攻击，造成了{}伤害\n"

                            player1turncost = player1turncost - 1
                            play_list.append(f"{user1skilldate['name']}减伤剩余{player1turncost}回合")
                            play_list.append(msg1.format(player1['道号'], player1_sh))
                            player2['气血'] = player2['气血'] - int(player1_sh * player2js)  # 玩家1的伤害 * 玩家2的减伤
                            play_list.append(f"{player2['道号']}剩余血量{player2['气血']}")
                            player1js = 1 - user1skillsh

                    elif user1skill_type == 4:  # 封印类技能
                        player1turncost = player1turncost - 1
                        skillmsg = get_persistent_skill_msg(player1['道号'], user1skilldate['name'], user1skillsh,
                                                            player1turncost)
                        play_list.append(skillmsg)
                        isCrit, player1_sh = get_turnatk(player1)  # 判定是否暴击
                        if isCrit:
                            msg1 = "{}发起会心一击，造成了{}伤害\n"
                        else:
                            msg1 = "{}发起攻击，造成了{}伤害\n"
                        play_list.append(msg1.format(player1['道号'], player1_sh))

                        player2['气血'] = player2['气血'] - int(player1_sh * player2js)  # 玩家1的伤害 * 玩家2的减伤
                        play_list.append(f"{player2['道号']}剩余血量{player2['气血']}")
                        if player1turncost == 0:  # 封印时间到
                            user2turnskip = True
                            user2buffturn = True


            else:  # 休息回合-1
                play_list.append(f"☆------{player1['道号']}动弹不得！------☆")
                if player1turncost > 0:
                    player1turncost -= 1
                if player1turncost == 0 and user1buffturn:
                    user1turnskip = True

        else:  # 没有技能的derB
            if user1turnskip:
                play_list.append(f"☆------{player1['道号']}的回合------☆")
                isCrit, player1_sh = get_turnatk(player1)  # 判定是否暴击
                if isCrit:
                    msg1 = "{}发起会心一击，造成了{}伤害\n"
                else:
                    msg1 = "{}发起攻击，造成了{}伤害\n"
                play_list.append(msg1.format(player1['道号'], player1_sh))
                player2['气血'] = player2['气血'] - player1_sh
                play_list.append(f"{player2['道号']}剩余血量{player2['气血']}")

            else:
                play_list.append(f"☆------{player1['道号']}动弹不得！------☆")

        if player2['气血'] <= 0:  # 玩家2气血小于0，结算
            play_list.append("{}胜利".format(player1['道号']))
            suc = f"{player1['道号']}"
            if isSql:
                XiuxianDateManage().update_user_hp_mp(player1['user_id'], int(player1['气血'] / (1 + user1hpbuff)),
                                                      int(player1['真元'] / (1 + user1mpbuff)))
                XiuxianDateManage().update_user_hp_mp(player2['user_id'], 1, int(player2['真元'] / (1 + user2mpbuff)))
            break

        if player1turncost < 0:  # 休息为负数，如果休息，则跳过回合，正常是0
            user1turnskip = False
            player1turncost += 1

        if player2_skil_open:  # 有技能
            if user2turnskip:  # 玩家2无需跳过回合
                play_list.append(f"☆------{player2['道号']}的回合------☆")
                user2hpconst, user2mpcost, user2skill_type, skillrate = get_skill_hp_mp_data(player2, user2skilldate)
                if player2turncost == 0:  # 没有持续性技能生效
                    player2js = 1
                    if isEnableUserSikll(player2, user2hpconst, user2mpcost, player2turncost,
                                         skillrate):  # 满足技能要求，#此处为技能的第一次释放
                        skillmsg, user2skillsh, player2turncost = get_skill_sh_data(player2, user2skilldate)
                        if user2skill_type == 1:  # 直接伤害类技能
                            play_list.append(skillmsg)
                            player1['气血'] = player1['气血'] - int(user2skillsh * player1js)  # 玩家2的伤害 * 玩家1的减伤
                            play_list.append(f"{player1['道号']}剩余血量{player1['气血']}")
                            player2 = calculate_skill_cost(player2, user2hpconst, user2mpcost)


                        elif user2skill_type == 2:  # 持续性伤害技能
                            play_list.append(skillmsg)
                            player1['气血'] = player1['气血'] - int(user2skillsh * player1js)  # 玩家2的伤害 * 玩家1的减伤
                            play_list.append(f"{player1['道号']}剩余血量{player1['气血']}")
                            player2 = calculate_skill_cost(player2, user2hpconst, user2mpcost)


                        elif user2skill_type == 3:  # buff类技能
                            user2buff_type = user2skilldate['bufftype']
                            if user2buff_type == 1:  # 攻击类buff
                                isCrit, player2_sh = get_turnatk(player2, user2skillsh)  # 判定是否暴击
                                if isCrit:
                                    msg2 = "{}发起会心一击，造成了{}伤害\n"
                                else:
                                    msg2 = "{}发起攻击，造成了{}伤害\n"

                                play_list.append(skillmsg)
                                play_list.append(msg2.format(player2['道号'], player2_sh))  # 玩家2的伤害 * 玩家1的减伤
                                player1['气血'] = player1['气血'] - int(player2_sh * player1js)
                                play_list.append(f"{player1['道号']}剩余血量{player1['气血']}")
                                player2 = calculate_skill_cost(player2, user2hpconst, user2mpcost)

                            elif user2buff_type == 2:  # 减伤类buff,需要在player2处判断
                                isCrit, player2_sh = get_turnatk(player2)  # 判定是否暴击
                                if isCrit:
                                    msg2 = "{}发起会心一击，造成了{}伤害\n"
                                else:
                                    msg2 = "{}发起攻击，造成了{}伤害\n"
                                play_list.append(skillmsg)
                                play_list.append(msg2.format(player2['道号'], player2_sh))  # 玩家2的伤害 * 玩家1的减伤
                                player1['气血'] = player1['气血'] - int(player2_sh * player1js)
                                play_list.append(f"{player1['道号']}剩余血量{player1['气血']}")
                                player2js = 1 - user2skillsh
                                player2 = calculate_skill_cost(player2, user2hpconst, user2mpcost)


                        elif user2skill_type == 4:  # 封印类技能
                            play_list.append(skillmsg)
                            player2 = calculate_skill_cost(player2, user2hpconst, user2mpcost)

                            if user2skillsh:  # 命中
                                user1turnskip = False
                                user1buffturn = False

                    else:  # 没放技能
                        isCrit, player2_sh = get_turnatk(player2)  # 判定是否暴击
                        if isCrit:
                            msg2 = "{}发起会心一击，造成了{}伤害\n"
                        else:
                            msg2 = "{}发起攻击，造成了{}伤害\n"
                        play_list.append(msg2.format(player2['道号'], player2_sh))
                        player1['气血'] = player1['气血'] - int(player2_sh * player1js)  # 玩家2的伤害 * 玩家1的减伤
                        play_list.append(f"{player1['道号']}剩余血量{player1['气血']}")


                else:  # 持续性技能判断,不是第一次
                    if user2skill_type == 2:  # 持续性伤害技能
                        player2turncost = player2turncost - 1
                        skillmsg = get_persistent_skill_msg(player2['道号'], user2skilldate['name'], user2skillsh,
                                                            player2turncost)
                        play_list.append(skillmsg)

                        isCrit, player2_sh = get_turnatk(player2)  # 判定是否暴击
                        if isCrit:
                            msg2 = "{}发起会心一击，造成了{}伤害\n"
                        else:
                            msg2 = "{}发起攻击，造成了{}伤害\n"

                        play_list.append(msg2.format(player2['道号'], player2_sh))  # 玩家2的伤害 * 玩家1的减伤,持续性伤害不影响普攻
                        player1['气血'] = player1['气血'] - int((user2skillsh + player2_sh) * player1js)
                        play_list.append(f"{player1['道号']}剩余血量{player1['气血']}")


                    elif user2skill_type == 3:  # buff类技能
                        user2buff_type = user2skilldate['bufftype']
                        if user2buff_type == 1:  # 攻击类buff
                            isCrit, player2_sh = get_turnatk(player2, user2skillsh)  # 判定是否暴击

                            if isCrit:
                                msg2 = "{}发起会心一击，造成了{}伤害\n"
                            else:
                                msg2 = "{}发起攻击，造成了{}伤害\n"
                            player2turncost = player2turncost - 1
                            play_list.append(f"{user2skilldate['name']}增伤剩余{player2turncost}回合")
                            play_list.append(msg2.format(player2['道号'], player2_sh))
                            player1['气血'] = player1['气血'] - int(player2_sh * player1js)  # 玩家2的伤害 * 玩家1的减伤
                            play_list.append(f"{player1['道号']}剩余血量{player1['气血']}")

                        elif user2buff_type == 2:  # 减伤类buff,需要在player2处判断
                            print("玩家2减伤" + str(player2js))
                            isCrit, player2_sh = get_turnatk(player2)  # 判定是否暴击
                            if isCrit:
                                msg2 = "{}发起会心一击，造成了{}伤害\n"
                            else:
                                msg2 = "{}发起攻击，造成了{}伤害\n"

                            player2turncost = player2turncost - 1
                            play_list.append(f"{user2skilldate['name']}减伤剩余{player2turncost}回合！")
                            play_list.append(msg2.format(player2['道号'], player2_sh))
                            player1['气血'] = player1['气血'] - int(player2_sh * player1js)  # 玩家1的伤害 * 玩家2的减伤
                            play_list.append(f"{player1['道号']}剩余血量{player1['气血']}")
                            player2js = 1 - user2skillsh

                    elif user2skill_type == 4:  # 封印类技能
                        player2turncost = player2turncost - 1
                        skillmsg = get_persistent_skill_msg(player2['道号'], user2skilldate['name'], user2skillsh,
                                                            player2turncost)
                        play_list.append(skillmsg)
                        isCrit, player2_sh = get_turnatk(player2)  # 判定是否暴击
                        if isCrit:
                            msg2 = "{}发起会心一击，造成了{}伤害\n"
                        else:
                            msg2 = "{}发起攻击，造成了{}伤害\n"
                        play_list.append(msg2.format(player2['道号'], player2_sh))

                        player1['气血'] = player1['气血'] - int(player2_sh * player1js)  # 玩家1的伤害 * 玩家2的减伤
                        play_list.append(f"{player1['道号']}剩余血量{player1['气血']}")

                        if player2turncost == 0:  # 封印时间到
                            user1turnskip = True
                            user1buffturn = True

            else:  # 休息回合-1
                play_list.append(f"☆------{player2['道号']}动弹不得！------☆")
                if player2turncost > 0:
                    player2turncost -= 1
                if player2turncost == 0 and user2buffturn:
                    user2turnskip = True
        else:  # 没有技能的derB
            if user2turnskip:
                play_list.append(f"☆------{player2['道号']}的回合------☆")
                isCrit, player2_sh = get_turnatk(player2)  # 判定是否暴击
                if isCrit:
                    msg2 = "{}发起会心一击，造成了{}伤害\n"
                else:
                    msg2 = "{}发起攻击，造成了{}伤害\n"
                play_list.append(msg2.format(player2['道号'], player2_sh))
                player1['气血'] = player1['气血'] - player2_sh
                play_list.append(f"{player1['道号']}剩余血量{player1['气血']}")

            else:
                play_list.append(f"☆------{player2['道号']}动弹不得！------☆")

        if player1['气血'] <= 0:  # 玩家2气血小于0，结算
            play_list.append("{}胜利".format(player2['道号']))
            suc = f"{player2['道号']}"
            if isSql:
                XiuxianDateManage().update_user_hp_mp(player1['user_id'], 1, int(player1['真元'] / (1 + user1mpbuff)))
                XiuxianDateManage().update_user_hp_mp(player2['user_id'], int(player2['气血'] / (1 + user2hpbuff)),
                                                      int(player2['真元'] / (1 + user2mpbuff)))
            break

        if player2turncost < 0:  # 休息为负数，如果休息，则跳过回合，正常是0
            user2turnskip = False
            player2turncost += 1

        if user1turnskip == False and user2turnskip == False:
            play_list.append('双方都动弹不得！')
            user1turnskip = True
            user2turnskip = True

        if player1['气血'] <= 0 or player2['气血'] <= 0:
            play_list.append("逻辑错误！！！")
            break

    return play_list, suc


def Boss_fight(player1: dict, boss: dict, type_in=2):
    """
    回合制战斗
    type_in : 1-切磋，不消耗气血、真元
              2-战斗，消耗气血、真元
    数据示例：
    {"user_id": None,"道号": None, "气血": None, "攻击": None, "真元": None, '会心':None, 'exp':None}
    """
    user1buffdate = UserBuffDate(player1['user_id'])  # 1号的buff信息
    user1mainbuffdata = user1buffdate.get_user_main_buff_data()
    user1hpbuff = user1mainbuffdata['hpbuff'] if user1mainbuffdata != None else 0
    user1mpbuff = user1mainbuffdata['mpbuff'] if user1mainbuffdata != None else 0

    # 有技能，则开启技能模式

    player1_skil_open = False
    if user1buffdate.get_user_sec_buff_data() != None:
        user1skilldate = user1buffdate.get_user_sec_buff_data()
        player1_skil_open = True

    play_list = []
    suc = None
    isSql = False
    if type_in == 2:
        isSql = True
    user1turnskip = True
    bossturnskip = True
    player1turncost = 0  # 先设定为初始值 0
    player1js = 1  # 减伤率
    boss['减伤'] = 1  # boss减伤率
    user1skillsh = 0

    user1buffturn = True
    bossbuffturn = True

    get_stone = 0
    sh = 0
    qx = boss['气血']
    bossnowstone = boss['stone']
    bossjs = boss['减伤']
    boss['会心'] = 30

    while True:
        msg1 = "{}发起攻击，造成了{}伤害\n"
        msg2 = "{}发起攻击，造成了{}伤害\n"
        if player1_skil_open:  # 是否开启技能
            if user1turnskip:  # 无需跳过回合
                print(player1turncost)
                play_list.append(f"☆------{player1['道号']}的回合------☆")
                user1hpconst, user1mpcost, user1skill_type, skillrate = get_skill_hp_mp_data(player1, user1skilldate)
                if player1turncost == 0:  # 没有持续性技能生效
                    player1js = 1  # 没有持续性技能生效,减伤恢复
                    if isEnableUserSikll(player1, user1hpconst, user1mpcost, player1turncost,
                                         skillrate):  # 满足技能要求，#此处为技能的第一次释放
                        skillmsg, user1skillsh, player1turncost = get_skill_sh_data(player1, user1skilldate)
                        if user1skill_type == 1:  # 直接伤害类技能
                            play_list.append(skillmsg)
                            player1 = calculate_skill_cost(player1, user1hpconst, user1mpcost)
                            boss['气血'] = boss['气血'] - int(user1skillsh * bossjs)  # 玩家1的伤害 * boss的减伤
                            play_list.append(f"{boss['name']}剩余血量{boss['气血']}")
                            sh += user1skillsh


                        elif user1skill_type == 2:  # 持续性伤害技能
                            play_list.append(skillmsg)
                            player1 = calculate_skill_cost(player1, user1hpconst, user1mpcost)
                            boss['气血'] = boss['气血'] - int(user1skillsh * bossjs)  # 玩家1的伤害 * 玩家2的减伤
                            play_list.append(f"{boss['name']}剩余血量{boss['气血']}")
                            sh += user1skillsh


                        elif user1skill_type == 3:  # buff类技能
                            user1buff_type = user1skilldate['bufftype']
                            if user1buff_type == 1:  # 攻击类buff
                                isCrit, player1_sh = get_turnatk(player1, user1skillsh)  # 判定是否暴击
                                if isCrit:
                                    msg1 = "{}发起会心一击，造成了{}伤害\n"
                                else:
                                    msg1 = "{}发起攻击，造成了{}伤害\n"
                                player1 = calculate_skill_cost(player1, user1hpconst, user1mpcost)
                                play_list.append(skillmsg)
                                play_list.append(msg1.format(player1['道号'], player1_sh))
                                boss['气血'] = boss['气血'] - int(player1_sh * bossjs)  # 玩家1的伤害 * 玩家2的减伤
                                play_list.append(f"{boss['name']}剩余血量{boss['气血']}")
                                sh += player1_sh


                            elif user1buff_type == 2:  # 减伤类buff,需要在player2处判断
                                isCrit, player1_sh = get_turnatk(player1)  # 判定是否暴击
                                if isCrit:
                                    msg1 = "{}发起会心一击，造成了{}伤害\n"
                                else:
                                    msg1 = "{}发起攻击，造成了{}伤害\n"

                                player1 = calculate_skill_cost(player1, user1hpconst, user1mpcost)
                                play_list.append(skillmsg)
                                play_list.append(msg1.format(player1['道号'], player1_sh))
                                boss['气血'] = boss['气血'] - int(player1_sh * bossjs)  # 玩家1的伤害 * 玩家2的减伤
                                play_list.append(f"{boss['name']}剩余血量{boss['气血']}")
                                player1js = 1 - user1skillsh
                                sh += player1_sh


                        elif user1skill_type == 4:  # 封印类技能
                            play_list.append(skillmsg)
                            player1 = calculate_skill_cost(player1, user1hpconst, user1mpcost)

                            if user1skillsh:  # 命中
                                bossturnskip = False
                                bossbuffturn = False

                    else:  # 没放技能，打一拳
                        isCrit, player1_sh = get_turnatk(player1)  # 判定是否暴击
                        if isCrit:
                            msg1 = "{}发起会心一击，造成了{}伤害\n"
                        else:
                            msg1 = "{}发起攻击，造成了{}伤害\n"
                        play_list.append(msg1.format(player1['道号'], player1_sh))
                        boss['气血'] = boss['气血'] - int(player1_sh * bossjs)  # 玩家1的伤害 * 玩家2的减伤
                        play_list.append(f"{boss['name']}剩余血量{boss['气血']}")
                        sh += player1_sh


                else:  # 持续性技能判断,不是第一次
                    if user1skill_type == 2:  # 持续性伤害技能
                        player1turncost = player1turncost - 1
                        skillmsg = get_persistent_skill_msg(player1['道号'], user1skilldate['name'], user1skillsh,
                                                            player1turncost)
                        play_list.append(skillmsg)
                        isCrit, player1_sh = get_turnatk(player1)  # 判定是否暴击
                        if isCrit:
                            msg1 = "{}发起会心一击，造成了{}伤害\n"
                        else:
                            msg1 = "{}发起攻击，造成了{}伤害\n"
                        play_list.append(msg1.format(player1['道号'], player1_sh))
                        # 玩家1的伤害 * 玩家2的减伤,持续性伤害不影响普攻
                        boss['气血'] = boss['气血'] - int((user1skillsh + player1_sh) * bossjs)
                        play_list.append(f"{boss['name']}剩余血量{boss['气血']}")
                        sh += player1_sh + user1skillsh


                    elif user1skill_type == 3:  # buff类技能
                        user1buff_type = user1skilldate['bufftype']
                        if user1buff_type == 1:  # 攻击类buff
                            isCrit, player1_sh = get_turnatk(player1, user1skillsh)  # 判定是否暴击

                            if isCrit:
                                msg1 = "{}发起会心一击，造成了{}伤害\n"
                            else:
                                msg1 = "{}发起攻击，造成了{}伤害\n"
                            player1turncost = player1turncost - 1
                            play_list.append(f"{user1skilldate['name']}增伤剩余:{player1turncost}回合")
                            play_list.append(msg1.format(player1['道号'], player1_sh))
                            boss['气血'] = boss['气血'] - int(player1_sh * bossjs)  # 玩家1的伤害 * 玩家2的减伤
                            play_list.append(f"{boss['name']}剩余血量{boss['气血']}")
                            sh += player1_sh


                        elif user1buff_type == 2:  # 减伤类buff,需要在player2处判断
                            isCrit, player1_sh = get_turnatk(player1)  # 判定是否暴击
                            if isCrit:
                                msg1 = "{}发起会心一击，造成了{}伤害\n"
                            else:
                                msg1 = "{}发起攻击，造成了{}伤害\n"

                            player1turncost = player1turncost - 1
                            play_list.append(f"减伤剩余{player1turncost}回合！")
                            play_list.append(msg1.format(player1['道号'], player1_sh))
                            boss['气血'] = boss['气血'] - int(player1_sh * bossjs)  # 玩家1的伤害 * 玩家2的减伤
                            play_list.append(f"{boss['name']}剩余血量{boss['气血']}")
                            player1js = 1 - user1skillsh
                            sh += player1_sh

                    elif user1skill_type == 4:  # 封印类技能
                        player1turncost = player1turncost - 1
                        skillmsg = get_persistent_skill_msg(player1['道号'], user1skilldate['name'], user1skillsh,
                                                            player1turncost)
                        play_list.append(skillmsg)
                        isCrit, player1_sh = get_turnatk(player1)  # 判定是否暴击
                        if isCrit:
                            msg1 = "{}发起会心一击，造成了{}伤害\n"
                        else:
                            msg1 = "{}发起攻击，造成了{}伤害\n"
                        play_list.append(msg1.format(player1['道号'], player1_sh))

                        boss['气血'] = boss['气血'] - int(player1_sh * bossjs)  # 玩家1的伤害 * 玩家2的减伤
                        play_list.append(f"{boss['name']}剩余血量{boss['气血']}")
                        sh += player1_sh
                        if player1turncost == 0:  # 封印时间到
                            bossturnskip = True
                            bossbuffturn = True

            else:  # 休息回合-1
                play_list.append(f"☆------{player1['道号']}动弹不得！------☆")
                if player1turncost > 0:
                    player1turncost -= 1
                if player1turncost == 0:
                    user1turnskip = True

        else:  # 没有技能的derB
            play_list.append(f"☆------{player1['道号']}的回合------☆")
            isCrit, player1_sh = get_turnatk(player1)  # 判定是否暴击
            if isCrit:
                msg1 = "{}发起会心一击，造成了{}伤害\n"
            else:
                msg1 = "{}发起攻击，造成了{}伤害\n"
            play_list.append(msg1.format(player1['道号'], player1_sh))
            boss['气血'] = boss['气血'] - player1_sh
            play_list.append(f"{boss['name']}剩余血量{boss['气血']}")
            sh += player1_sh

        if boss['气血'] <= 0:  # boss气血小于0，结算
            play_list.append("{}胜利".format(player1['道号']))
            suc = f"{player1['道号']}"
            get_stone = bossnowstone
            if isSql:
                XiuxianDateManage().update_user_hp_mp(player1['user_id'], int(player1['气血'] / (1 + user1hpbuff)),
                                                      int(player1['真元'] / (1 + user1mpbuff)))

            break

        if player1turncost < 0:  # 休息为负数，如果休息，则跳过回合，正常是0
            user1turnskip = False
            player1turncost += 1

            # 没有技能的derB
        if bossturnskip:
            play_list.append(f"☆------{boss['name']}的回合------☆")
            isCrit, boss_sh = get_turnatk(boss)  # 判定是否暴击
            if isCrit:
                msg2 = "{}发起会心一击，造成了{}伤害\n"
            else:
                msg2 = "{}发起攻击，造成了{}伤害\n"
            play_list.append(msg2.format(boss['name'], boss_sh))
            player1['气血'] = player1['气血'] - (boss_sh * player1js)
            play_list.append(f"{player1['道号']}剩余血量{player1['气血']}")

        else:
            play_list.append(f"☆------{boss['name']}动弹不得！------☆")

        if player1['气血'] <= 0:  # 玩家2气血小于0，结算
            play_list.append("{}胜利".format(boss['name']))
            suc = f"{boss['name']}"

            get_stone = int(bossnowstone * (sh / qx))
            boss['stone'] = bossnowstone - get_stone

            if isSql:
                XiuxianDateManage().update_user_hp_mp(player1['user_id'], 1, int(player1['真元'] / (1 + user1mpbuff)))

            break

        if player1['气血'] <= 0 or boss['气血'] <= 0:
            play_list.append("逻辑错误！！！")
            break

    return play_list, suc, boss, get_stone


def get_turnatk(player, buff=0):
    isCrit = False
    turnatk = int(round(random.uniform(0.95, 1.05), 2) * (player['攻击'] * (buff + 1)))  # 攻击波动,buff是攻击buff
    if random.randint(0, 100) <= player['会心']:  # 会心判断
        turnatk = int(turnatk * 1.5)
        isCrit = True
    return isCrit, turnatk


def isEnableUserSikll(player, hpcost, mpcost, turncost, skillrate):  # 是否满足技能释放条件
    skill = False
    if turncost < 0:  # 判断是否进入休息状态
        return skill

    if player['气血'] > hpcost and player['真元'] >= mpcost:  # 判断血量、真元是否满足
        if random.randint(0, 100) <= skillrate:  # 随机概率释放技能
            skill = True
    return skill


def get_skill_hp_mp_data(player, secbuffdata):
    hpcost = int(secbuffdata['hpcost'] * player['气血']) if secbuffdata['hpcost'] != 0 else 0
    mpcost = int(secbuffdata['mpcost'] * player['exp']) if secbuffdata['mpcost'] != 0 else 0
    return hpcost, mpcost, secbuffdata['type'], secbuffdata['rate']


def calculate_skill_cost(player, hpcost, mpcost):
    player['气血'] = player['气血'] - hpcost  # 气血消耗
    player['真元'] = player['真元'] - mpcost  # 真元消耗

    return player


def get_persistent_skill_msg(username, skillname, sh, turn):
    if sh == True:
        return f"{username}的封印技能：{skillname}，剩余回合：{turn}！"
    return f"{username}的持续性技能：{skillname}，造成{sh}伤害，剩余回合：{turn}！"


def get_skill_sh_data(player, secbuffdata):
    skillmsg = ''
    if secbuffdata['type'] == 1:  # 连续攻击类型
        turncost = -secbuffdata['turncost']
        isCrit, turnatk = get_turnatk(player)
        atkvalue = secbuffdata['atkvalue']  # 列表
        skillsh = 0
        atkmsg = ''
        for value in atkvalue:
            atkmsg += f"{int(value * turnatk)}伤害、"
            skillsh += int(value * turnatk)

        if turncost == 0:
            turnmsg = '！'
        else:
            turnmsg = f"，休息{secbuffdata['turncost']}回合！"

        if isCrit:
            skillmsg = f"{player['道号']}发动技能：{secbuffdata['name']}，消耗气血{int(secbuffdata['hpcost'] * player['气血']) if secbuffdata['hpcost'] != 0 else 0}点、真元{int(secbuffdata['mpcost'] * player['exp']) if secbuffdata['mpcost'] != 0 else 0}点，{secbuffdata['desc']}并且发生了会心一击，造成{atkmsg[:-1]}{turnmsg}"
        else:
            skillmsg = f"{player['道号']}发动技能：{secbuffdata['name']}，消耗气血{int(secbuffdata['hpcost'] * player['气血']) if secbuffdata['hpcost'] != 0 else 0}点、真元{int(secbuffdata['mpcost'] * player['exp']) if secbuffdata['mpcost'] != 0 else 0}点，{secbuffdata['desc']}造成{atkmsg[:-1]}{turnmsg}"

        return skillmsg, skillsh, turncost

    elif secbuffdata['type'] == 2:  # 持续伤害类型
        turncost = secbuffdata['turncost']
        isCrit, turnatk = get_turnatk(player)
        skillsh = int(secbuffdata['atkvalue'] * player['攻击'])  # 改动
        atkmsg = ''
        if isCrit:
            skillmsg = f"{player['道号']}发动技能：{secbuffdata['name']}，消耗气血{int(secbuffdata['hpcost'] * player['气血']) if secbuffdata['hpcost'] != 0 else 0}点、真元{int(secbuffdata['mpcost'] * player['exp']) if secbuffdata['mpcost'] != 0 else 0}点，{secbuffdata['desc']}并且发生了会心一击，造成{skillsh}点伤害，持续{turncost}回合！"
        else:
            skillmsg = f"{player['道号']}发动技能：{secbuffdata['name']}，消耗气血{int(secbuffdata['hpcost'] * player['气血']) if secbuffdata['hpcost'] != 0 else 0}点、真元{int(secbuffdata['mpcost'] * player['exp']) if secbuffdata['mpcost'] != 0 else 0}点，{secbuffdata['desc']}造成{skillsh}点伤害，持续{turncost}回合！"

        return skillmsg, skillsh, turncost

    elif secbuffdata['type'] == 3:  # 持续buff类型
        turncost = secbuffdata['turncost']
        skillsh = secbuffdata['buffvalue']
        atkmsg = ''
        if secbuffdata['bufftype'] == 1:
            skillmsg = f"{player['道号']}发动技能：{secbuffdata['name']}，消耗气血{int(secbuffdata['hpcost'] * player['气血']) if secbuffdata['hpcost'] != 0 else 0}点、真元{int(secbuffdata['mpcost'] * player['exp']) if secbuffdata['mpcost'] != 0 else 0}点，{secbuffdata['desc']}攻击力增加{skillsh}倍，持续{turncost}回合！"
        elif secbuffdata['bufftype'] == 2:
            skillmsg = f"{player['道号']}发动技能：{secbuffdata['name']}，消耗气血{int(secbuffdata['hpcost'] * player['气血']) if secbuffdata['hpcost'] != 0 else 0}点、真元{int(secbuffdata['mpcost'] * player['exp']) if secbuffdata['mpcost'] != 0 else 0}点，{secbuffdata['desc']}获得{skillsh * 100}%的减伤，持续{turncost}回合！"

        return skillmsg, skillsh, turncost

    elif secbuffdata['type'] == 4:  # 封印类技能
        turncost = secbuffdata['turncost']
        if random.randint(0, 100) <= secbuffdata['success']:  # 命中
            skillsh = True
            skillmsg = f"{player['道号']}发动技能：{secbuffdata['name']}，消耗气血{int(secbuffdata['hpcost'] * player['气血']) if secbuffdata['hpcost'] != 0 else 0}点、真元{int(secbuffdata['mpcost'] * player['exp']) if secbuffdata['mpcost'] != 0 else 0}点，使对手动弹不得,{secbuffdata['desc']}持续{turncost}回合！"
        else:  # 未命中
            skillsh = False
            skillmsg = f"{player['道号']}发动技能：{secbuffdata['name']}，消耗气血{int(secbuffdata['hpcost'] * player['气血']) if secbuffdata['hpcost'] != 0 else 0}点、真元{int(secbuffdata['mpcost'] * player['exp']) if secbuffdata['mpcost'] != 0 else 0}点，{secbuffdata['desc']}但是被对手躲避！"

        return skillmsg, skillsh, turncost



import json
import os
from pathlib import Path

from .xiuxian2_handle import XiuxianDateManage


READPATH = Path() / "data" / "xiuxian" / "功法"
class BuffJsonDate:

    def __init__(self):
        """json文件路径"""
        self.mainbuff_jsonpath = READPATH / "主功法.json"
        self.secbuff_jsonpath = READPATH / "神通.json"
        self.gfpeizhi_jsonpath = READPATH / "功法概率设置.json"
    
    def get_main_buff(self, id):
        return readf(self.mainbuff_jsonpath)[str(id)]
    
    def get_sec_buff(self, id):
        return readf(self.secbuff_jsonpath)[str(id)]
    
    def get_gfpeizhi(self):
        return readf(self.gfpeizhi_jsonpath)
    
    
class UserBuffDate:
    
    def __init__(self, user_id):
        """用户Buff数据"""
        self.user_id = user_id
        self.BuffInfo = get_user_buff(self.user_id)
        
    
    
    def get_user_main_buff_data(self):
        try:
            mainbuffdata = BuffJsonDate().get_main_buff(str(self.BuffInfo.main_buff))
        except:
            mainbuffdata = None
        return mainbuffdata
    
    def get_user_sec_buff_data(self):
        try:
            secbuffdata = BuffJsonDate().get_sec_buff(str(self.BuffInfo.sec_buff))
        except:
            secbuffdata = None
        return secbuffdata


def get_main_info_msg(id):
    mainbuff = BuffJsonDate().get_main_buff(id)
    hpmsg = f"提升{round(mainbuff['hpbuff'] * 100, 0)}%气血" if mainbuff['hpbuff'] != 0 else ''
    mpmsg = f"提升{round(mainbuff['mpbuff'] * 100, 0)}%真元" if mainbuff['mpbuff'] != 0 else ''
    atkmsg = f"提升{round(mainbuff['atkbuff'] * 100, 0)}%攻击力" if mainbuff['atkbuff'] != 0 else ''
    ratemsg = f"提升{round(mainbuff['ratebuff'] * 100, 0)}%修炼速度" if mainbuff['ratebuff'] != 0 else ''
    msg = f"{mainbuff['name']}：{hpmsg},{mpmsg},{atkmsg},{ratemsg}。"
    return mainbuff, msg


def get_user_buff(user_id):
    BuffInfo = XiuxianDateManage().get_user_buff_info(user_id)
    if BuffInfo == None:
        XiuxianDateManage().initialize_user_buff_info(user_id)
        return XiuxianDateManage().get_user_buff_info(user_id)
    else:
        return BuffInfo


def readf(FILEPATH):
        with open(FILEPATH, "r", encoding="UTF-8") as f:
            data = f.read()
        return json.loads(data)
    
def get_sec_msg(secbuffdata):
    if secbuffdata == None:
        msg = "无"
        return msg
    hpmsg = f"，消耗当前血量{int(secbuffdata['hpcost'] * 100)}%" if secbuffdata['hpcost'] != 0 else ''
    mpmsg = f"，消耗真元{int(secbuffdata['mpcost'] * 100)}%" if secbuffdata['mpcost'] != 0 else ''

    if secbuffdata['type'] == 1:
        shmsg = ''
        for value in secbuffdata['atkvalue']:
            shmsg += f"{value}倍、"
        if secbuffdata['turncost'] == 0:
            msg = f"攻击{len(secbuffdata['atkvalue'])}次，造成{shmsg[:-1]}伤害{hpmsg}{mpmsg}，释放概率：{secbuffdata['rate']}%"
        else:
            msg = f"连续攻击{len(secbuffdata['atkvalue'])}次，造成{shmsg[:-1]}伤害{hpmsg}{mpmsg}，休息{secbuffdata['turncost']}回合，释放概率：{secbuffdata['rate']}%"
    elif secbuffdata['type'] == 2:
        msg = f"持续伤害，造成{secbuffdata['atkvalue']}倍攻击力伤害{hpmsg}{mpmsg}，持续{secbuffdata['turncost']}回合，释放概率：{secbuffdata['rate']}%"
    elif secbuffdata['type'] == 3:
        if secbuffdata['bufftype'] == 1:
            msg = f"增强自身，提高{secbuffdata['buffvalue']}倍攻击力{hpmsg}{mpmsg}，持续{secbuffdata['turncost']}回合，释放概率：{secbuffdata['rate']}%"
        elif secbuffdata['bufftype'] == 2:
            msg = f"增强自身，提高{secbuffdata['buffvalue'] * 100}%减伤率{hpmsg}{mpmsg}，持续{secbuffdata['turncost']}回合，释放概率：{secbuffdata['rate']}%"
    elif secbuffdata['type'] == 4:
        msg = f"封印对手{hpmsg}{mpmsg}，持续{secbuffdata['turncost']}回合，释放概率：{secbuffdata['rate']}%，命中成功率{secbuffdata['success']}%"

    return msg


    






from nonebot.adapters.onebot.v11 import (
    PRIVATE_FRIEND,
    Bot,
    GROUP,
    Message,
    MessageEvent,
    GroupMessageEvent,
)
from datetime import datetime
from nonebot import get_bot, on_command, on_regex, require
from ..xiuxian2_handle import XiuxianDateManage, XiuxianJsonDate, OtherSet
from ..xiuxian_config import XiuConfig, JsonConfig
from ..utils import check_user
from ..data_source import jsondata
from ..read_buff import BuffJsonDate, UserBuffDate, get_main_info_msg, get_user_buff, get_sec_msg
from nonebot.permission import SUPERUSER
from nonebot.params import CommandArg, RegexGroup
from ..player_fight import Player_fight
from ..utils import send_forward_msg, Txt2Img, data_check_conf
from ..cd_manager import add_cd, check_cd, cd_msg

buffinfo = on_command("我的功法", priority=5)
out_closing = on_command("出关", aliases={"灵石出关"}, priority=5)
mind_state = on_command("我的状态", priority=5)
qc = on_command("切磋", priority=5)
buff_help = on_command("功法帮助", priority=5)
sql_message = XiuxianDateManage()  # sql类

__buff_help__ = f"""
功法帮助信息:
指令：
1、我的功法：查看自身功法信息
2、切磋，at对应人员，不会消耗气血
""".strip()

@buff_help.handle()
async def _(bot: Bot, event: GroupMessageEvent):
    """修仙帮助"""
    await data_check_conf(bot, event)
    msg = __buff_help__
    await buff_help.finish(msg)

@qc.handle()
async def _(bot: Bot, event: GroupMessageEvent, args: Message = CommandArg()):
    """切磋，不会掉血"""
    await data_check_conf(bot, event)
    isUser, user_info, msg = check_user(event)
    if not isUser:
        await qc.finish(msg, at_sender=True)
    user_id = user_info.user_id
    
    give_qq = None  # 艾特的时候存到这里
    for arg in args:
        if arg.type == "at":
            give_qq = arg.data.get("qq", "")
    if give_qq:
        if give_qq == str(user_id):
            await qc.finish("道友不会左右互搏之术！")
                
        if cd := check_cd(event, '切磋'):
            # 如果 CD 还没到 则直接结束
            await qc.finish(cd_msg(cd), at_sender=True)
        player1 = {"user_id": None, "道号": None, "气血": None,
                   "攻击": None, "真元": None, '会心': None, '防御': 0, 'exp': 0}
        player2 = {"user_id": None, "道号": None, "气血": None,
                   "攻击": None, "真元": None, '会心': None, '防御': 0, 'exp': 0}
        user1 = sql_message.get_user_real_info(user_id)
        user2 = sql_message.get_user_real_info(give_qq)
        player1['user_id'] = user1.user_id
        player1['道号'] = user1.user_name
        player1['气血'] = user1.hp
        player1['攻击'] = user1.atk
        player1['真元'] = user1.mp
        player1['会心'] = 1
        player1['exp'] = user1.exp

        player2['user_id'] = user2.user_id
        player2['道号'] = user2.user_name
        player2['气血'] = user2.hp
        player2['攻击'] = user2.atk
        player2['真元'] = user2.mp
        player2['会心'] = 1
        player2['exp'] = user2.exp
        
        result, victor = Player_fight(player1, player2, 1)
        add_cd(event, 300, '切磋')
        await send_forward_msg(bot, event, '决斗场', bot.self_id, result)
        await qc.finish(f"获胜的是{victor}")
    else:
        await qc.finish("没有对方的信息！")

@out_closing.handle()
async def _(bot: Bot, event: GroupMessageEvent):
    """出关"""
    await data_check_conf(bot, event)
    user_type = 0  # 状态0为无事件
    isUser, user_info, msg = check_user(event)
    if not isUser:
        await out_closing.finish(msg)
    user_id = user_info.user_id
    user_mes = sql_message.get_user_message(user_id)  # 获取用户信息
    level = user_mes.level
    use_exp = user_mes.exp
    hp_speed = 25
    mp_speed = 50

    max_exp = (
            int(OtherSet().set_closing_type(level)) * XiuConfig().closing_exp_upper_limit
    )  # 获取下个境界需要的修为 * 1.5为闭关上限
    user_get_exp_max = int(max_exp) - use_exp

    if user_get_exp_max < 0:
        # 校验当当前修为超出上限的问题，不可为负数
        user_get_exp_max = 0

    now_time = datetime.now()
    user_cd_message = sql_message.get_user_cd(user_id)

    if user_cd_message is None:
        # 不存在用户信息
        await out_closing.finish("没有查到道友的信息，修炼发送【闭关】，进入修炼状态！", at_sender=True)

    elif user_cd_message.type == 0:
        # 用户状态为0
        await out_closing.finish("道友现在什么都没干呢~", at_sender=True)

    elif user_cd_message.type == 1:
        # 用户状态为1
        in_closing_time = datetime.strptime(
            user_cd_message.create_time, "%Y-%m-%d %H:%M:%S.%f"
        )  # 进入闭关的时间
        exp_time = (
                OtherSet().date_diff(now_time, in_closing_time) // 60
        )  # 闭关时长计算(分钟) = second // 60
        level_rate = sql_message.get_root_rate(user_mes.root_type)  # 灵根倍率
        realm_rate = jsondata.level_data()[level]["spend"]  # 境界倍率
        mainbuffdata = UserBuffDate(user_id).get_user_main_buff_data()
        mainbuffratebuff = mainbuffdata['ratebuff'] if mainbuffdata != None else 0#功法修炼倍率
        exp = int(
            exp_time * XiuConfig().closing_exp * level_rate * realm_rate * (1 + mainbuffratebuff)
        )  # 本次闭关获取的修为

        if exp >= user_get_exp_max:
            # 用户获取的修为到达上限
            sql_message.in_closing(user_id, user_type)
            sql_message.update_exp(user_id, user_get_exp_max)
            sql_message.update_power2(user_id)  # 更新战力

            result_msg, result_hp_mp = OtherSet().send_hp_mp(user_id, int(exp * hp_speed), int(exp*mp_speed))
            sql_message.update_user_attribute(user_id, result_hp_mp[0], result_hp_mp[1], int(result_hp_mp[2] / 10))
            await out_closing.finish(
                "闭关结束，本次闭关到达上限，共增加修为：{}{}{}".format(user_get_exp_max, result_msg[0], result_msg[1]), at_sender=True
            )
        else:
            # 用户获取的修为没有到达上限

            if str(event.message) == "灵石出关":
                user_stone = user_mes.stone  # 用户灵石数
                if exp <= user_stone:
                    exp = exp * 2
                    sql_message.in_closing(user_id, user_type)
                    sql_message.update_exp(user_id, exp)
                    sql_message.update_ls(user_id, int(exp / 2), 2)
                    sql_message.update_power2(user_id)  # 更新战力

                    result_msg, result_hp_mp = OtherSet().send_hp_mp(user_id, int(exp * hp_speed), int(exp * mp_speed))
                    sql_message.update_user_attribute(user_id, result_hp_mp[0], result_hp_mp[1],
                                                      int(result_hp_mp[2] / 10))
                    await out_closing.finish(
                        "闭关结束，共闭关{}分钟，本次闭关增加修为：{}，消耗灵石{}枚{}{}".format(exp_time, exp, int(exp / 2),
                                                                      result_msg[0], result_msg[1]), at_sender=True
                    )
                else:
                    exp = exp + user_stone
                    sql_message.in_closing(user_id, user_type)
                    sql_message.update_exp(user_id, exp)
                    sql_message.update_ls(user_id, user_stone, 2)
                    sql_message.update_power2(user_id)  # 更新战力
                    result_msg, result_hp_mp = OtherSet().send_hp_mp(user_id, int(exp * hp_speed), int(exp * mp_speed))
                    sql_message.update_user_attribute(user_id, result_hp_mp[0], result_hp_mp[1],
                                                      int(result_hp_mp[2] / 10))
                    await out_closing.finish(
                        "闭关结束，共闭关{}分钟，本次闭关增加修为：{}，消耗灵石{}枚{}{}".format(exp_time, exp, user_stone,
                                                                  result_msg[0], result_msg[1]), at_sender=True
                    )
            else:
                sql_message.in_closing(user_id, user_type)
                sql_message.update_exp(user_id, exp)
                sql_message.update_power2(user_id)  # 更新战力
                result_msg, result_hp_mp = OtherSet().send_hp_mp(user_id, int(exp * hp_speed), int(exp * mp_speed))
                sql_message.update_user_attribute(user_id, result_hp_mp[0], result_hp_mp[1], int(result_hp_mp[2] / 10))
                await out_closing.finish(
                    "闭关结束，共闭关{}分钟，本次闭关增加修为：{}{}{}".format(exp_time, exp, result_msg[0],
                                                          result_msg[1]), at_sender=True
                )

    elif user_cd_message.type == 2:
        await out_closing.finish("悬赏令事件进行中，请输入【悬赏令结算】结束！", at_sender=True)

@mind_state.handle()
async def _(bot: Bot, event: GroupMessageEvent):
    """我的状态信息。"""
    await data_check_conf(bot, event)
    isUser, user_msg, msg = check_user(event)
    if not isUser:
        await mind_state.finish(msg)
    user_id = user_msg.user_id


    if user_msg.hp is None or user_msg.hp == 0 or user_msg.hp == 0:
        sql_message.update_user_hp(user_id)
    user_msg = sql_message.get_user_real_info(user_id)
    
    level_rate = sql_message.get_root_rate(user_msg.root_type)  # 灵根倍率
    realm_rate = jsondata.level_data()[user_msg.level]["spend"]  # 境界倍率
    mainbuffdata = UserBuffDate(user_id).get_user_main_buff_data()
    mainbuffratebuff = mainbuffdata['ratebuff'] if mainbuffdata != None else 0
    mainhpbuff = mainbuffdata['hpbuff'] if mainbuffdata != None else 0
    mainmpbuff = mainbuffdata['mpbuff'] if mainbuffdata != None else 0
    user = f"""道号：{user_msg.user_name}
气血：{user_msg.hp}/{int((user_msg.exp/2) * (1 + mainhpbuff))}
真元：{user_msg.mp}/{int((user_msg.exp) * (1 + mainmpbuff))}
攻击：{user_msg.atk}
攻击修炼：{user_msg.atkpractice}级(提升攻击力{user_msg.atkpractice * 10}%)
修炼效率：{int((level_rate * realm_rate) * (1 + mainbuffratebuff)  * 100)}%
"""

    await mind_state.finish(user, at_sender=True)


@buffinfo.handle()
async def _(bot: Bot, event: GroupMessageEvent):
    await data_check_conf(bot, event)

    isUser, user_info, msg = check_user(event)
    if not isUser:
        await buffinfo.finish(msg)
    user_id = user_info.user_id
    mainbuffdata = UserBuffDate(user_id).get_user_main_buff_data()
    if mainbuffdata != None:
        s, mainbuffmsg = get_main_info_msg(str(get_user_buff(user_id).main_buff))
    else:
        mainbuffmsg = ''
    secbuffdata = UserBuffDate(user_id).get_user_sec_buff_data()
    secbuffmsg = get_sec_msg(secbuffdata) if get_sec_msg(secbuffdata) != '无' else ''
    msg = f"""
道友的主功法：{mainbuffdata["name"] if mainbuffdata != None else '无'}
{mainbuffmsg}
道友的神通：{secbuffdata["name"] if secbuffdata != None else '无'}
{secbuffmsg}
"""
    await buffinfo.finish(msg, at_sender=True)




import json
import os
from pathlib import Path

configkey = [
    "LEVLECOST", 
    "等级建设度", 
    "发放宗门资材", 
    "每日宗门任务次上限", 
    "宗门任务完成cd", 
    "宗门任务刷新cd", 
    "宗门任务",
    "宗门主功法参数",
    "宗门神通参数"
    ]
CONFIG = {
    "LEVLECOST" : {
    #攻击修炼的灵石消耗
    '0':1000,
    '1':2000,
    '2':4000,
    '3':8000,
    '4':16000,
    '5':32000,
    '6':50000,
    '7':50000,
    '8':50000,
    '9':50000,
    '10':100000,
    '11':100000,
    '12':100000,
    '13':100000,
    '14':100000,
    '15':100000,
    '16':100000,
    '17':100000,
    '18':100000,
    '19':100000,
    '20':200000,
    '21':200000,
    '22':200000,
    '23':200000,
    '24':200000,
    '25':0,
    },
    "等级建设度":500000,#决定宗门修炼上限等级的参数，500万贡献度每级
    "发放宗门资材":{
        "时间":"11-12",#定时任务发放宗门资材，每日11-12点根据 对应宗门贡献度的 * 倍率 发放资材
        "倍率":1,#倍率
    },
    "每日宗门任务次上限":3,
    "宗门任务完成cd":1800,#宗门任务每次完成间隔，单位秒
    "宗门任务刷新cd":300,#宗门任务刷新间隔，单位秒
    "宗门主功法参数":{
        "获取消耗的资材":600000,#最终消耗会乘档位
        "获取消耗的灵石":30000,#最终消耗会乘档位
        "获取到功法的概率":100,
        "建设度":1000000,#建设度除以此参数，一共10档（10档目前无法配置,对应天地玄黄人上下）
        "学习资材消耗":600000,#最终消耗会乘档位
    },
    "宗门神通参数":{
        "获取消耗的资材":600000,#最终消耗会乘档位
        "获取消耗的灵石":30000,#最终消耗会乘档位
        "获取到神通的概率":100,
        "建设度":1000000,#建设度除以此参数，一共10档（10档目前无法配置,对应天地玄黄人上下）
        "学习资材消耗":600000,#最终消耗会乘档位
    },
    "宗门任务":{
        #type=1：需要扣气血，type=2：需要扣灵石
        #cost：消耗，type=1时，气血百分比，type=2时，消耗灵石
        #give：给与玩家当前修为的百分比修为
        #sect：给与所在宗门 储备的灵石，同时会增加灵石 * 10 的建设度
        "巡逻周边":{
            "desc":"宗门附近有人在捣乱，请道友去周边震慑宵小吧",
            "type":1,
            "cost":0.1,
            "give":0.002,
            "sect":2000,
        },
        "寻找宝物":{
            "desc":"宗门现在需要宝物：捆仙锁，请道友去寻来交给宗门吧",
            "type":2,
            "cost":3000,
            "give":0.006,
            "sect":9000,
        },
      "外出探查": {
         "desc": "宗门东北方有烟雾弥漫，请道友前去查看情况",
         "type": 1,
         "cost": 0.1,
         "give": 0.002,
         "sect": 2000
      },
      "抓捕劣徒": {
         "desc": "有弟子坠于铜臭，借钱抛洒，债台高筑，请道友将其抓回惩戒",
         "type": 1,
         "cost": 0.2,
         "give": 0.004,
         "sect": 4000
      },
      "狩猎邪修": {
         "desc": "传言山外村庄有邪修抢夺灵石，请道友下山为民除害",
         "type": 1,
         "cost": 0.35,
         "give": 0.007,
         "sect": 7000
      },
      "查抄窝点": {
         "desc": "有少量弟子不在金银阁消费，私自架设小型窝点，请道友前去查抄",
         "type": 1,
         "cost": 0.25,
         "give": 0.005,
         "sect": 5000
      },
      "搜查物品": {
         "desc": "宗门有弟子私下收藏铁锅和锤子烧火做饭，请道友前往搜查",
         "type": 1,
         "cost": 0.1,
         "give": 0.002,
         "sect": 2000
      },
      "九转仙丹": {
         "desc": "山门将开，宗门急缺一批药草熬制九转丹，请道友下山购买",
         "type": 2,
         "cost": 5000,
         "give": 0.01,
         "sect": 15000
      },
      "劝离凡俗": {
         "desc": "有凡俗漂泊至山外，虽有仙缘，但无仙份，请道友筹措一笔盘缠，助其回乡，重入红尘",
         "type": 2,
         "cost": 1000,
         "give": 0.002,
         "sect": 3000
      },
      "仗义疏财": {
         "desc": "在宗门外见到师弟欠了别人灵石被追打催债，请道友帮助其还清赌债",
         "type": 2,
         "cost": 4000,
         "give": 0.008,
         "sect": 12000
      },
      "红尘寻宝": {
         "desc": "山下一月一度的市场又开张了，其中虽凡物较多，但是请道友慷慨解囊，为宗门购买一些蒙尘奇宝",
         "type": 2,
         "cost": 6000,
         "give": 0.012,
         "sect": 18000
      }
    }
}

def get_config():
    try:
        config = readf()
        for key in configkey:
            if key not in list(config.keys()):
                config[key] = CONFIG[key]
        savef(config)
    except:
        config = CONFIG
        savef(config)
    return config

CONFIGJSONPATH = Path(__file__).parent
FILEPATH = CONFIGJSONPATH / 'config.json'
def readf():
    with open(FILEPATH, "r", encoding="UTF-8") as f:
        data = f.read()
    return json.loads(data)


def savef(data):
    data = json.dumps(data, ensure_ascii=False, indent=3)
    savemode = "w" if os.path.exists(FILEPATH) else "x"
    with open(FILEPATH, mode=savemode, encoding="UTF-8") as f:
        f.write(data)
        f.close
    return True
